// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  crystal growth group                                                  ==
// ==                                                                        ==
// ==  Stiftung caesar                                                       ==
// ==  Ludwig-Erhard-Allee 2                                                 ==
// ==  53175 Bonn                                                            ==
// ==  germany                                                               ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  http://www.caesar.de/cg/AMDiS                                         ==
// ==                                                                        ==
// ============================================================================

/** \file VtkWriter.h */

#ifndef AMDIS_VTKWRITER_H
#define AMDIS_VTKWRITER_H

#include <fstream>

#include "BasisFunction.h"
#include "DataCollector.h"

namespace AMDiS {

  class VtkWriter
  {
  public:
    VtkWriter(::std::vector<DataCollector*> *dc,
	      int level = -1,
	      Flag traverseFlag = Mesh::CALL_LEAF_EL)
      : dc_(dc),
        level_(level),
        traverseFlag_(traverseFlag)
    {
      degree_ = (*dc_)[0]->getFeSpace()->getBasisFcts()->getDegree();    
      dim_ = (*dc_)[0]->getMesh()->getDim();
    };  


    /** \brief
     * Writes a ParaView-VTK file.
     */
    int writeFile(const char *name);


    /** \brief
     * Adds a new entry to a ParaView animation file.
     */
    int updateAnimationFile(::std::string valueFilename,
			    ::std::vector< ::std::string > *paraViewAnimationFrames,
			    const char *animationFilename);
  protected:
    /** \brief
     * 
     */
    void writeVertexCoords(::std::ofstream &file);


    /** \brief
     * 
     */
    void writeVertexValues(::std::ofstream &file, int componentNo);


    /** \brief
     * 
     */
    void writeConnectivity(::std::ofstream &file);

    /** \brief
     *
     */
    void writeConnectivity_dim2_degree2(::std::ofstream &file);


    /** \brief
     *
     */
    inline void writeCoord(::std::ofstream &file, WorldVector<double> coord) {
      for (int i = 0; i < Global::getGeo(WORLD); i++) {
	file << " " << ::std::scientific << coord[i];
      }
      for (int i = Global::getGeo(WORLD); i < 3; i++) {
	file << " 0.0";
      }
      file << ::std::endl;
    }

  private:
    /**
     *
     */
    ::std::vector<DataCollector*> *dc_;


    /** \brief
     * Level information for traversing the mesh.
     */
    int level_;


    /** \brief
     * Flags for traversing the mesh.
     */
    Flag traverseFlag_;


    /** \brief
     * 
     */
    int degree_;
    

    /** \brief
     * 
     */
    int dim_;
  };
}

#endif
