#include "AMDiS.h"

using namespace std;
using namespace AMDiS;

// ===========================================================================
// ===== function definitions ================================================
// ===========================================================================

/** \brief
 * RHS function
 */
class F : public AbstractFunction<double, WorldVector<double> >
{
public:
  MEMORY_MANAGED(F);

  F(int degree) : AbstractFunction<double, WorldVector<double> >(degree) {};

  /** \brief
   * Implementation of AbstractFunction::operator().
   */
  const double& operator()(const WorldVector<double>& x) const {
    static double result = 0.0;
    result = -2 * x[0];
    return result;
  };
};

/** \brief
 * boundary
 */
class G : public AbstractFunction<double, WorldVector<double> >
{
public:
  MEMORY_MANAGED(G);

  /** \brief
   * Implementation of AbstractFunction::operator().
   */
  const double& operator()(const WorldVector<double>& x) const {
    static double result = 10000.0;
    return result;
  };
};
// ===========================================================================
// ===== main program ========================================================
// ===========================================================================

int main(int argc, char* argv[])
{
  FUNCNAME("bunny main");

  // ===== check for init file =====
  TEST_EXIT(argc == 2)("usage: bunny initfile\n");

  // ===== init parameters =====
  Parameters::init(false, argv[1]);

  // ===== create projection =====
  WorldVector<double> ballCenter;
  ballCenter.set(0.0);
  NEW BallProject(1, 
		  VOLUME_PROJECTION, 
		  ballCenter, 
		  1.0);

  // ===== create and init the scalar problem ===== 
  ProblemScal bunny("bunny");
  bunny.initialize(INIT_ALL);

  // === create adapt info ===
  AdaptInfo *adaptInfo = NEW AdaptInfo("bunny->adapt");

  // === create adapt ===
  AdaptStationary *adapt = NEW AdaptStationary("bunny->adapt",
					       &bunny,
					       adaptInfo);
  
  // ===== add boundary conditions =====
  //bunny.addDirichletBC(1111, NEW G);

  // ===== create matrix operator =====
  Operator matrixOperator(Operator::MATRIX_OPERATOR,
			  bunny.getFESpace());
  matrixOperator.addSecondOrderTerm(NEW Laplace_SOT);
  bunny.addMatrixOperator(&matrixOperator);

  // ===== create rhs operator =====
  Operator rhsOperator(Operator::VECTOR_OPERATOR,
		       bunny.getFESpace());

  int degree = bunny.getFESpace()->getBasisFcts()->getDegree();

  rhsOperator.addZeroOrderTerm(NEW CoordsAtQP_ZOT(NEW F(degree)));
  bunny.addVectorOperator(&rhsOperator);

  // ===== start adaption loop =====
  adapt->adapt();

  bunny.writeFiles(adaptInfo, true);

  return 0;
}


